#include <avr/io.h>
#include "usart.h"
#include <avr/interrupt.h>
#include "lcd.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define F_CPU 16000000
#define NR_OF_BUTTONS 12

#define UP_BUTTON 11
#define DOWN_BUTTON 10

int buttons[NR_OF_BUTTONS] = { PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PB1, PB2, PB3, PB4 };
int buttons_last_value[NR_OF_BUTTONS];

int current_millis;
int last_millis;

volatile char message[100];
int current_button = 0;
int last_button = 0;
volatile uint8_t index = 0;
int number_of_presses = 0;

#define MENU_CALL 1
#define MENU_MESSAGE 2
#define MENU 3
#define MENU_MESSAGE_INS_NUM 4

int menu_place = 0;

volatile char phone_number[11] = "";
volatile int phone_number_index = 0;

void USART0_printInt(int i)
{
                char debug[2];
                debug[0] = (char)(((int)'0')+i);
                debug[1] = '\0';
                USART0_print(debug);
}


ISR(USART0_RX_vect)
{
  char a = UDR0;
  if( a == '>' )
  {
    USART0_print(message);
    USART0_print("\r\x1A\0");
  }
  USART1_transmit(a);
}


ISR(PCINT0_vect)
{
        cli();
        uint8_t i = 0;
        for(i=0;i<8;i++)
        {
          if( !(PINA & (1 << buttons[i])))
          {
            if( current_millis - last_millis > 20 && buttons_last_value[i] != 1 )
            {
                if( menu_place == MENU_CALL   )
                {
                  phone_number[phone_number_index++] =  (char)(((int)'0')+i);
                  phone_number[phone_number_index] = '\0'; 
                  LCD_writeInstr(LCD_INSTR_returnHome);
                  LCD_print(phone_number);
                  if( phone_number_index > 10 ) phone_number_index = 0;
                }
                  else if( menu_place == MENU_MESSAGE_INS_NUM )
                  {
                    phone_number[phone_number_index++] =  (char)(((int)'0')+i);
                    phone_number[phone_number_index] = '\0'; 
                    LCD_writeInstr(LCD_INSTR_returnHome);
                    LCD_writeInstr(LCD_INSTR_nextLine);
                    LCD_print(phone_number);
                    if( phone_number_index > 10 ) phone_number_index = 0;
                  }
                else if ( menu_place == MENU_MESSAGE )
                {
                  current_button = i;
                  if( current_millis - last_millis < 100 && current_button == last_button )
                  {
                    if( number_of_presses == 3 ) number_of_presses = 0;
                    else number_of_presses++; 

                    message[index-1] = (char)('a' + 4 * i + number_of_presses);
                  }
                  else
                  {
                    number_of_presses = 0;
                    message[index] = (char)('a' + 4 * i);
                    index++;
                  }
                  last_button = current_button;
                  message[index] = '\0';
//                  USART0_transmit(message);
                  LCD_writeInstr(LCD_INSTR_returnHome);
                  LCD_print(message);
                }
            }
            buttons_last_value[i] = 1;
          }
          else
          {
            buttons_last_value[i] = 0;
          }
        }
        last_millis = current_millis;
        sei();
}


ISR(PCINT1_vect)
{
        cli();
        int i = 0;
        for(i=8;i<=NR_OF_BUTTONS;i++)
        {
          if( !(PINB & (1 << buttons[i])))
          {
            if( current_millis - last_millis > 20 && buttons_last_value[i] != 1 )
            {
                if( menu_place == MENU_CALL && i < 10   )
                {
                  phone_number[phone_number_index++] =  (char)(((int)'0')+i);
                  phone_number[phone_number_index] = '\0'; 
                  LCD_writeInstr(LCD_INSTR_returnHome);
                  LCD_print(phone_number);
                  if( phone_number_index > 10 ) phone_number_index = 0;
                }
                if( i == DOWN_BUTTON && (current_millis - last_millis > 20) )
                {
                    if( menu_place == MENU_CALL )
                    {
                      char call_command[15];
                      strncpy(call_command,"ATD",3);
                      strncpy(call_command+3,phone_number,strlen(phone_number));
                      call_command[13] = '\r';
                      call_command[14] = '\0';
//                      call_command[15] = '\0';
                      USART0_print(call_command);
                   }
                    else
                    {
                     LCD_writeInstr(LCD_INSTR_clearDisplay);
                     LCD_writeInstr(LCD_INSTR_returnHome);
                     LCD_putChar('C');
                     menu_place = MENU_CALL;
                     phone_number[0] = '\0';
                     phone_number_index = 0;
                    }
                }
                else
                if( i == UP_BUTTON && (current_millis - last_millis > 20) )
                {

                    if( menu_place == MENU_MESSAGE )
                    {
                      menu_place = MENU_MESSAGE_INS_NUM;
                      LCD_writeInstr(LCD_INSTR_clearDisplay);
                      LCD_writeInstr(LCD_INSTR_returnHome);
                      LCD_print("Insert number\0");

                    }
                    else if( menu_place == MENU_MESSAGE_INS_NUM )
                    {

                      USART0_print("at+cmgf=1\r\0");
                      USART0_print("at+cmgs=");
                      USART0_transmit((char)34);
                      USART0_print("+4\0");
                      USART0_print(phone_number);
                      USART0_transmit((char)34);
                      USART0_transmit('\r');

//                      USART0_print("at+cmgf=1\r\0");
//                      USART0_print("at+cmgs=\"+40743002107\"\r");
                      //message[index+1] = '\r';
                      //message[index+2] = '\x1A';
                      //message[index+3] = '\0';
                      //char message_debug[] = "Salut\r\x1A\0";
                      //USART0_print(message_debug);
                    }
                    else
                    {
                    LCD_writeInstr(LCD_INSTR_clearDisplay);
                    LCD_writeInstr(LCD_INSTR_returnHome);
                    LCD_putChar('M');
                    menu_place = MENU_MESSAGE;                    
                    phone_number[0] = '\0';
                    phone_number_index = 0;
                    message[0] = '\0';
                    index = 0;
                    }
                }
                // else
                // {
            }

            buttons_last_value[i] = 1;
          }
          else
          {
            buttons_last_value[i] = 0;
          }
        }
        last_millis = current_millis;
        sei();
}

ISR(TIMER0_COMPA_vect)
{
    current_millis++;
}

void init_timer()
{
    TCCR0A |= (1 << WGM01); OCR0A = 200;
    TCCR0B |= (1 << CS02);
    TIMSK0 |= (1 << OCIE0A);
}

void setup()
{
    cli();
    PCICR |= (1 << PCIE0);
	PCMSK0 = 0xFF;

	PCICR |= (1 << PCIE1);
	PCMSK1 = 0x1F;

    int i = 0;
    for(i=0;i<8;i++)
    {
        DDRA &= ~(1 << buttons[i]);
        PORTA |= (1 << buttons[i]);
    }
    for(i=8;i<=NR_OF_BUTTONS;i++)
    {
        DDRB &= ~( 1 << buttons[i]);
        PORTB |= (1 << buttons[i]);
    }

	USART0_init();
    USART1_init();
	init_timer();
    
    LCD_init();	
    LCD_print(" Mesaj");
    LCD_writeInstr(LCD_INSTR_nextLine);
    LCD_print(" Apel");
    sei();
}

int main()
{
	setup();
	
	while(1)
	{
		
	}
}
